//
// Copyright © 2021 Kris Nóva <kris@nivenly.com>
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//   ███╗   ██╗ █████╗ ███╗   ███╗██╗
//   ████╗  ██║██╔══██╗████╗ ████║██║
//   ██╔██╗ ██║███████║██╔████╔██║██║
//   ██║╚██╗██║██╔══██║██║╚██╔╝██║██║
//   ██║ ╚████║██║  ██║██║ ╚═╝ ██║███████╗
//   ╚═╝  ╚═══╝╚═╝  ╚═╝╚═╝     ╚═╝╚══════╝
//

package naml

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/client-go/kubernetes"
)

// Deployable is an interface that can be implemented
// for deployable applications.
type Deployable interface {

	// Install will attempt to install in Kubernetes
	Install(client kubernetes.Interface) error

	// Uninstall will attempt to uninstall in Kubernetes
	Uninstall(client kubernetes.Interface) error

	// Meta returns a NAML Meta structure which embed Kubernetes *metav1.ObjectMeta
	Meta() *AppMeta

	// Objects will return the runtime objects defined for each application
	Objects() []runtime.Object
}

type AppMeta struct {
	Description string
	metav1.ObjectMeta
}
